require 'livecode'
require 'vector'
require 'box'

-- TODOs:
---- santa

INTERNAL = false

function love.load()
    love.filesystem.load("pictures.lua")()
    love.filesystem.load("items.lua")()
    love.filesystem.load("combi.lua")()
    love.filesystem.load("snaps.lua")()
    love.filesystem.load("rituals.lua")()
    love.filesystem.load("background.lua")()
    love.filesystem.load("music.lua")()
    love.filesystem.load("intro.lua")()

    initGame()
end

function love.livereload()
    -- resetGame()

end

function initGame()
    screenSize = Vector(1920, 1080)
    if INTERNAL then
        screenScale = Vector(0.75, 0.75)
    else
        screenScale = Vector(1,1)
    end
    viewSize = screenSize ^ screenScale
    -- love.window.setMode(viewSize.x, viewSize.y)
    -- love.window.setFullscreen(true, "desktop")
    love.window.setMode(viewSize.x, viewSize.y, {fullscreen = true, fullscreentype = "desktop"})
    -- if not INTERNAL then
    -- end

    local realRes = Vector(love.graphics.getWidth(), love.graphics.getHeight())
    screenScale = (realRes/screenSize)


    initBackground()
    initCombi()
    initSnaps()
    initItems()
    initMusic()
    initSfx()
    initRituals()
    initIntro()

    font = love.graphics.newFont("fnt/AABK.ttf",100)
    headerFont = love.graphics.newFont("fnt/AABK.ttf",162)

    resetGame()
end

function resetGame()
    resetPics()
    resetCombi()
    resetItems()
    resetRituals()
    resetMusic()
    resetIntro()

    readyToReset = false
    screenShake = Vector(0,0)
end

function love.update(dt)
    updateIntro(dt)
    updateRituals(dt)
    updateItems(dt)
    updatePics(dt)
    updateBackground(dt)
    updatePenta(dt)
end

function love.draw()
    love.graphics.push()

    love.graphics.translate(screenShake.x, screenShake.y)
    love.graphics.scale(screenScale.x, screenScale.y)

    if Rituals.distortion then
        love.graphics.push()
        love.graphics.translate(screenSize.x * 0.5, screenSize.y * 0.5)
        -- local sc = (1 + math.cos(t)) * 0.1
        love.graphics.scale(1 + Rituals.distAmount, 1 + Rituals.distMax - Rituals.distAmount)
        love.graphics.translate(-screenSize.x * 0.5, -screenSize.y * 0.5)
    end

    drawBackground()
    drawPenta()
    drawItems()
    drawPics()
    drawForeground()

    if Rituals.distortion then
        love.graphics.pop()
    end
    drawRituals()


    drawIntro()

    love.graphics.pop()
end


function love.mousepressed(x,y)
    -- local mpos = Vector(love.mouse.getX() / screenScale.x, love.mouse.getY() / screenScale.y)
    -- print(mpos.x..", "..mpos.y)

    if readyToReset then
        resetGame()
    end

    if Intro.active then
        introClicked()
    end
end


function love.keypressed(key)
    if key == "escape" then
        love.event.push("quit")
        return
    end

    if key == "f1" then
        local fs = love.window.getFullscreen()
        love.window.setMode(viewSize.x, viewSize.y, {fullscreen = not fs, fullscreentype = "desktop"})
        local realRes = Vector(love.graphics.getWidth(), love.graphics.getHeight())
        screenScale = (realRes/screenSize)
        return
    end

    if INTERNAL then
        if key == " " then
            launchSantaRitual()
        end

        if key == "s" then
            launchDevilRitual()
        end

        if key == "g" then
            launchGoatRitual()
        end
    end

    if readyToReset then
        resetGame()
    end

    if Intro.active then
        introClicked()
    end
end